;**********************************************************************
;      PO Metal detector using PIC12F1840                             *
;                                                                     *
;    Filename:                  metal1.asm                            *
;    Date:                      06.08.2017                            *
;    File Version:              2.00                                  *
;                                                                     *
;                                   4 operation modes                 *
;                mode0: general mode with ferrite rejection           *
;                mode1: gradually faster beeping as target is closer  *
;                mode2: without ferrite rejection                     *
;                mode3: silent mode, LED indication only              *
;                                       Double push changes mode!     *
;                                   short push: on                    *
;                                   long push: off                    *
;                                                                     *
;                                                                     *
;   Author:Teemo Berkis                                               *
;       www.digiwood.ee                                               *
;                                                                     *
;    Written and assembled under MPLAB X IDE Version 2.20             *
;                                MPASM Version 5.58                   *
;                                                                     *
;                                                                     *
;      diskriminatsioon mode works!                                   *
;      ported to 12F1572                                              *
;	                                                              *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
; Pinout connections for PIC12F1572 or PIC12F1840
;                            ___ ___
;                      Vdd+ -|1 ^ 8|- Vss-
;             mosfet drive  -|2   7|- Piezo1
;           measuring input -|3   6|- LED output
;         pushbutton switch -|4   5|- Piezo2
;                            -------
;**********************************************************************
;   Basic description of some things in program.
;    timer0 - mosfet pulse timing. Both ON duration, and OFF duration, using interrupts.
;    timer1 - does all the tricky business, measures the duration of the first 20 oscillation periods, then is swithed over
;             to the gate mode and measures rest of the oscillation decay.
;    timer2 - sound, using interrupts
;
;   
;**********************************************************************

	list	p=12F1572
	radix	dec
	include	"p12f1572.inc"	
	
		errorlevel	-302	; Dont complain about BANK 1 Registers during assembly

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================
 ;Internal osc.

    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF  & _BOREN_NSLEEP & _CLKOUTEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_ON & _BORV_LO & _LVP_OFF

;*************************  Register Definition  *********************
			UDATA_SHR
flowcounter			res		1	;program flow counter for interrupts.
soundflow			res		1	;sound sequence counter
f_resultL			res		1	;fereq measurement value(duration of first 20 sine wave periods)
f_resultH			res		1	;updated every 2ms
t_resultL			res		1	;decay time measurement value
t_resultH			res		1	;updated every 2ms
periodcount			res		1	;stores the number of detected oscillations, after every pulse
ReferenceL			res		1	;result of adaptive reference calculation
ReferenceH			res		1   ;(decay time reference)
fReference			res		2   ;frequency reference
tempL				res		1   ;general temp variables
tempH				res		1
beeptemp			res		1	;used in 0.5ms delay and 2ms delay
MainState1			res		1   ;Main mode state
flags				res		1	;various flags for communication between interrupts and main, and between main and sub
#define	soundflag	flags,0
#define	dataready	flags,1
#define	datareadyF	flags,2
#define	dataready2	flags,3
#define	dataready2F	flags,4
#define	soundflag2	flags,5
#define	AdjustVrefFlag	flags,6


buf1			UDATA		0x20
Counter1			res		1	;used in adaptive reference calculation
sum                 res		4	;used in adaptive reference calculation
IndexW				res		1	;index for writing to circular buffer
IndexR				res		1	;index for reading from circular buffer
ToneRef             res     2   ;sound tone reference
double_p_temp		res		1	;used in double push detector
CircularBuffer		res		64  ;32x2byte
							;79 bytes total block


buf2    		UDATA		0xA0	;second circular buffer 32 2Byte samples,for storing f_result values
IndexW2				res		1	;index for writing to circular buffer
IndexR2				res		1	;index for reading from circular buffer
f_result2           res     4   ;temporary variable for adding
CircularBuffer2		res		64	;32x2byte

moving_avg_3		UDATA			;third circular buffer 16 2Byte samples, used in moving average calculation for t_result
IndexW2_3			res		1	;index for writing to circular buffer
IndexR2_3			res		1	;index for reading from circular buffer
Addtmp_3			res		1	;extra byte for adding
CircularBuffer2_3	res		32	;16x2byte

various     UDATA
periodresult		res		1   ;final detected period count for one pulse
temp1				res		1	;used in button press detection

results2        UDATA
fresultfiltered32	res		2	;fereq measurement value(duration of first 20 sine wave periods)updated every 32ms
tresultfiltered32	res		2	;decay time measurement value updated every 32ms
koguja              res     2   
Sound               res     1   ;sound tone
SoundCounter1       res     1   ;beep duration counter

delay1		UDATA
temp2               res		1	;used in 250mS delay
temp3               res		1	;used in 250ms delay
counter4            res		1	;used in startup soundref counter

;************************* Output devices Definition   *********************
MOSFET			EQU		RA5		;MOSFET drive connection port	(active high)
LED1			EQU		RA1		;ver1:RA1 (ver2 board:RA2) Led contact
Piezo1			EQU		RA0		;piezo buzzer contacts
Piezo2			EQU		RA2     ;ver1:RA2 (ver2 board:RA1)

;************************* Input devices Definition   *********************
push_button 	EQU		RA3		;Button connection
MEASURING_INPUT	EQU		RA4		;Measuring input. DO NOT change! Changing it here will not change comparator input port.

;************************* Constants Definition   *********************
FetPulseTime	EQU		.136  	;MOSFET pulse width in microseconds (default.136)(range  20-255)(actual value=n+4us)
PauseTime		EQU		.115	;n*16=[us] 115*16=1840us  (range 2-255)
				;total	 2000us = 2ms period
sensitivity		EQU		.18		;min difference between reference and current value that causes "beep"
								;smaller value is higher sensitivity, default=18
CompRef			EQU		.27		;n for start comparator reference voltage:Vref=n*1,024/32(range 0-31))
								;actual value is adapted by circuit when it is powered up.
Maxmodes        EQU     .4      ;number of working modes (double push changes mode)
;button constants
shortpush_max		EQU		.25			;short push maximum duraton*20 =[ms], longer than this is long push
double_push_min		EQU		.1			;min wait between double push pushes [ms]/20, button release debounce
double_push_max		EQU		.25			;max wait between pulses [ms]/20 ,if longer than this pushes count as separate single pulses.
;****************************************************************
;Beginning of program
;****************************************************************
RESET_VECT  CODE    0x0000
    	PAGESEL START
    	GOTO    START

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------

ISR		CODE	0x0004
	
		btfss	INTCON,TMR0IF		;check for timer0 interrupt
		goto	OtherInt
Timer0int                           ;timer0 interrupt
		btfsc	flowcounter,0
		goto	Start_pulse 		;if flowcounter bit0, happens this
		btfsc	flowcounter,1
		goto	Stop_pulse			;if flowcounter bit1, happens this
OtherInt
		banksel	PIR2
		btfsc	PIR2,C1IF
		goto	ComparatorInterrupt
        banksel PIR1
		btfsc	PIR1,TMR2IF
		goto	Timer2Interrupt     ;to sound

        bcf     INTCON,GIE          ;just in case
		retfie	;(never reach this)

; INTERRUPT SERVICE ROUTINE SUBROUTINES
		CODE

Start_pulse						;Starts mosfet pulse
        movlw   b'11000010'     ; configure Timer0:
                ; --0-----          timer mode (T0CS = 0), internal instruction clock(8 MHz)
                ; ----0---          prescaler assigned to Timer0 (PSA = 0)
                ; -----010          prescale = 8
        banksel OPTION_REG      ;   -> increment TMR0 every 1 us
        movwf   OPTION_REG				;delay with timer0
        movlw   FetPulseTime
        comf    WREG,f          ;Nii saame ige vrtuse lespoole loendava taimeri jaoks	
		banksel TMR0
		movwf	TMR0			;load timer with FetPulseTime, starting timer
		bcf		INTCON,TMR0IF	;clear timer0 interrupt flag
		;---------------------------------------------
		banksel T1CON			;
		bcf		T1CON,TMR1ON			;timer1 stop
		bcf		T1GCON,TMR1GE
		;-----------------------save result (from previous pulse)-----------------------------
		btfsc	dataready		;test if data was successfully copied, if not then skip update.
		goto	skp_upd
		banksel TMR1L
		movf	TMR1L,w
		movwf	t_resultL
		movf	TMR1H,w
		movwf	t_resultH
		movf	periodcount,w
		banksel	periodresult
		movwf	periodresult	
		bsf		dataready		;set data ready flag			
skp_upd

		;-----------------------clear timer1---------------------------
		banksel	TMR1L
		clrf	TMR1L
		clrf	TMR1H		
		;----------------------------------------------------------------
		banksel	LATA
		bsf		LATA,MOSFET		;Turn on mosfet
		clrf	flowcounter
		bsf		flowcounter,1	;next step nr1,  Stop_pulse, set up for next timer0 interrupt
		;-----------------------comparator interrupt setup------------
;		bcf		INTCON,PEIE		;
		banksel	PIE2
		bcf		PIE2,C1IE		;comparator interrupt disable
		banksel	PIR2
		bcf		PIR2,C1IF		;clear comparator interrupt flag	
		;--------------------------------------------------------------
        ;------sound things--------------------
        banksel SoundCounter1
        incf    SoundCounter1,f
        btfss   SoundCounter1,4
        bra     skipclear
        bcf     soundflag2      ;enough of sound
        clrf    SoundCounter1
skipclear
        btfss   soundflag       ;test if sound is requested
        retfie
        clrf    SoundCounter1
        bsf     soundflag2      ;make sound
        bcf     soundflag       ;let free the soundflag

	retfie


Stop_pulse		
		;----------------------------------------------
		banksel	TMR0
		btfss	TMR0,2			;let timer0 count extra 4us, to compensate out possible other (sound) interrupts
		bra		$-1
		banksel LATA
		bcf		LATA,MOSFET		;turn off MOSFET!
		;------------------------mosfet turnoff delay--------
		call	rtrn2			;1us delay
		call	rtrn2			;1us delay
		;------------------------timer1 kima ---------------------
		banksel T1CON
		movlw	b'01000001'		;
				;'01------'		timer1 clock source is system clock 32MHz
				;'--00----'		prescale off(1:1)
				;'----xxx-'		ebaolulised
				;'-------1'		timer1 enable TMR1ON
		movwf	T1CON			;seadistatud
		;-----------------------comparator interrupt setup------------
		bsf		INTCON,PEIE		;
		banksel	PIE2
		bsf		PIE2,C1IE		;comparator interrupt enable
		banksel	PIR2
		bcf		PIR2,C1IF		;clear comparator interrupt flag			
		;-----------------------timer0 setup----------
		clrf	flowcounter
		bsf		flowcounter,0	;next step: nr0, ehk Start_Pulse
		movlw   b'11000110'     ; configure Timer0:
                ; --0-----          timer mode (T0CS = 0), sisemine instruktsiooni kell(8 MHz)
                ; ----0---          prescaler assigned to Timer0 (PSA = 0)
                ; -----110          prescale = 128
        banksel OPTION_REG      ;   -> increment TMR0 every 8 us
        movwf   OPTION_REG	
        movlw   PauseTime
        comf    WREG,f          ;Nii saame ige vrtuse lespoole loendava taimeri jaoks
		banksel TMR0
		movwf	TMR0			;laadime taimerisse PauseTime, seega taimer kima
		bcf		INTCON,TMR0IF	;clear timer0 interrupt flag,
		;----------------------counter------------------	
		clrf	periodcount

	retfie

ComparatorInterrupt
		banksel	PIR2
		bcf		PIR2,C1IF		;clear comparator interrupt flag
		;-------------------
		incf	periodcount,f
		;------------------
		movlw	.19
		subwf	periodcount,w	;check for 19-periodcount
		btfsc	STATUS,Z
		goto	cycle19

		movlw	.20
		subwf	periodcount,w	;check for 20-periodcount
		btfsc	STATUS,Z
		goto	cycle20

		btfss	AdjustVrefFlag	;check for comparator voltage reference adjustment mode
	retfie
		movlw	.240	;if  timer0 >240 at the moment, then decrease comparator ref voltage( by DAC) by one.
		banksel TMR0
		subwf	TMR0,w
		btfss	STATUS,C		;when W < TMR0 then C=1
	retfie
		banksel DACCON1
		decf	DACCON1,f
	retfie
cycle19		;----------------activate timer1 gate function so that timer stops at next rising edge (without stopping timer now) 
		banksel T1GCON
		bsf		T1GCON,T1GPOL
		movlw	b'01111010'		;
				;'0-------'		;gate function disabled TMR1GE
				;'-1------'		;gate active high
				;'--11----'		;gate single pulse and toggle mode
				;'------10'		;gate input from comparator
				;'----1---'		;t1ggo
		movwf	T1GCON			;
		bcf		T1GCON,T1GPOL
		bsf		T1GCON,T1GPOL	;force start gate circuit
		bsf		T1GCON,TMR1GE	;switch to gate control
	retfie
cycle20
		;-----------------------save result, this is duration of first 20 oscillation  periods.-----------------------------
		btfsc	datareadyF		;test if data was successfully copied by main program, if not then skip update.
		goto	skp_upd2
		banksel TMR1L
		movf	TMR1L,w
		movwf	f_resultL
		movf	TMR1H,w
		movwf	f_resultH
		bsf		datareadyF		;set data ready flag
skp_upd2
		;-----------------------clear timer1---------------------------
		clrf	TMR1L
		clrf	TMR1H	
		;--------------------start timer1, new settings---------------
		banksel T1GCON
		movlw	b'10000010'		;
				;'1-------'		;gate function enabled
				;'-0------'		;gate active low
				;'------10'		;gate input from comparator
		movwf	T1GCON			;
		banksel T1CON
		movlw	b'01000001'		;
				;'01------'		timer1 clock source is system clock 32MHz
				;'--00----'		prescale off(1:1)
				;'----xxx-'		ebaolulised
				;'-------1'		timer1 enable TMR1ON
		movwf	T1CON			;seadistatud
		;-------------------disable comparator interrupt--------------------
;		bcf		INTCON,PEIE		
	retfie

;-------------------------Timer2 interrupt, sound generation--------------
Timer2Interrupt			;
		banksel	PIR1
		bcf		PIR1,TMR2IF		;clear timer2 interrupt flag

		btfss	soundflag2
		bra 	vaikus      ;silence

;        banksel soundflow
        movfw   soundflow
        brw
        bra    soundflow0
        bra    soundflow1
        bra    soundflow2
        bra    soundflow3


soundflow0
        call    toon1
        incf    soundflow,f
        goto    polaarsus1
soundflow1
        call    toon1
        incf    soundflow,f
        goto    polaarsus2
soundflow2
        call    toon2
        incf    soundflow,f
        goto    polaarsus1
soundflow3
        call    toon2
        clrf    soundflow
        goto    polaarsus2

toon1   ;---basic tone about 630 Hz------
		movlw	.80            ;timer2 period
		banksel PR2
		movwf	PR2
        return

toon2   ;---variable tone-------------
        banksel Sound
        movf    Sound,W
   		banksel PR2
        movwf   PR2
        return

polaarsus1     ;---set speaker polarity
		banksel	LATA
		bcf		LATA,Piezo1
		bsf		LATA,Piezo2
		retfie
polaarsus2    ;---set speaker polarity
		banksel	LATA
		bcf		LATA,Piezo2
		bsf		LATA,Piezo1
		retfie
vaikus         ;silence
        banksel	LATA
		bcf		LATA,Piezo1
		bcf		LATA,Piezo2
        retfie


;***************************************************************************************
;
; MAIN PROGRAM
;
;***************************************************************************************
PGM		CODE		
START	;-------------------------general initialisation-----------------
        PAGESEL _2ms
		call	_2ms
		banksel OSCCON
		movlw	b'11110000'
		movwf	OSCCON			;configure oscillator to internal 32MHz
		banksel PORTA
		clrf	PORTA			;init portA
		banksel LATA
		clrf	LATA			;init port data latch
		banksel ANSELA
		clrf	ANSELA			;all digital I/O
		bsf		ANSELA,MEASURING_INPUT	;set comparator input analog
 		banksel TRISA
      	movlw	b'00111111'		;Set TRISA port: 0 =output, 1=input, all to input first
		movwf	TRISA	   	    ;
		banksel LATA
		bcf		LATA,MOSFET	
		banksel	TRISA
		bcf		TRISA,MOSFET	;MOSFETi to output
		bcf		TRISA,Piezo1	;piezo to output
		bcf		TRISA,Piezo2
		bcf		TRISA,LED1		;led ots vljundiks
	
		;------------------- init variables ------------------------------------
        clrf    flags
		clrf	flowcounter
        clrf    soundflow
        clrf    MainState1
        banksel SoundCounter1
        clrf    SoundCounter1
		banksel	Counter1
		clrf	Counter1
        banksel IndexW
		clrf	IndexW
        banksel ReferenceL
		clrf	ReferenceL
		clrf	ReferenceH
        banksel sum
		clrf	sum
		clrf	sum+1
		clrf	sum+2
        clrf    sum+3
		banksel	IndexW2
		clrf	IndexW2

        banksel Sound
        movlw   .150
        movwf   Sound

        banksel koguja
        movlw   b'11111111'
        movwf   koguja
        movlw   b'01111111'
        movwf   koguja+1
		;--------------------alive led-------------
		banksel LATA
		bsf		LATA,LED1
		;----------------------DAC setup-------------------------------------
		banksel	DACCON0			;
		movlw	b'11000000'		;positive ref Vdd
		movwf	DACCON0			;
		movlw	CompRef			;ref voltage for comparator, defined in constants section
		movwf	DACCON1			;

		;--------------------------------------	
		bsf		flowcounter,0	;jrgmine samm nr0, ehk Start_Pulse
		banksel INTCON			;enable interrupts
		movlw 	b'10100000'		
				; 1-------		general interrupt enable
				; --1-----		timer0 interrupt enable
		movwf	INTCON
		;----------------------------------------------------------------------
		bsf		INTCON,TMR0IF	;begin sending pulses, generate interrupt with soft, hakkame impulsse saatma, selleks tekitame kunstliku interrupti
		call	_250mS
		;---------------------setup comparator-------------------------------
		banksel	CM1CON0
		movlw	b'10010101'		;
				;'1-------'		;comparator enable
				;'--0-----'		;comparator output internal only
				;'---1----'		;output inverted
				;'------0-'		;hysteresis disabled
				;'-------1'		;sync with timer1
		movwf	CM1CON0			;
		movlw	b'10010001'		;
				;'1-------'		;rising edge interrupt
				;'--01----'		;+input connects to internal DAC Voltage reference
				;'-------1'		;-input connects to C1IN1- (RA4) 
		movwf	CM1CON1			;
		;-----------------------config sound-------------------
		call	confTimer2


		;-----------------------startup beep, same time auto adjust the vref--------------------
		bsf		soundflag		;sound on
		bsf		AdjustVrefFlag	;vref adjustment mode on
		call	_250mS
		bsf		soundflag		;sound on
		call	_250mS
		bsf		soundflag		;sound on
		call	_250mS
		bsf		soundflag		;sound on
		bcf		AdjustVrefFlag	;vref adjustment mode off
		;---------------init buffer-----------------------------
		call	init_buffer

        ;--------------initial MODE selection-----------------
        banksel MainState1
        movlw   .0      ;default mode

        movwf   MainState1

;****************************************************************
;* MAIN                                                         *
;****************************************************************
Main	;main loop
        banksel MainState1
        movfw   MainState1
        brw
        bra     MainMode0
        bra     MainMode1
        bra     MainMode2
        bra     MainMode3
        nop
        nop
        nop
MainMode0  ;discriminating mode, standard on/off sound
		btfss	dataready		;wait for data to become ready
		goto	bpress			;if data not ready skip calculations
		btfss	datareadyF		;check for data to be ready
		goto	bpress			;if data not ready skip calculations
		;--------------------------main calculations----------------------
		call	moving_average_3	;filter f_result
		call	nonferro			;nonferro block, free to comment out if not needed.
		call	fast_reference		;moving mode calculate new reference values
		call	calc
		bcf		dataready
		bcf		datareadyF
		bra 	bpress

MainMode1 ;gradual beeping(distance indication), now with discrimination
		btfss	dataready		;wait for data to become ready
		goto	bpress			;if data not ready goto button press detection instead
		btfss	datareadyF		;check for data to be ready
		goto	bpress			;if data not ready goto button press detection instead
                                ;else do calculations
;		call	nonferro			;nonferro block, free to comment out if not needed.
		call	fast_reference		;moving mode, calculate new reference values
        call    calc32
		bcf		dataready
		bcf		datareadyF
		bra 	bpress

MainMode2 ;discriminating mode less filtered
		btfss	dataready		;wait for data to become ready
		goto	bpress			;if data not ready goto button press detection instead
		btfss	datareadyF		;check for data to be ready
		goto	bpress			;if data not ready goto button press detection instead
                                ;else do calculations
;		call	moving_average_3	;filter f_result
;		call	nonferro			;nonferro block, free to comment out if not needed.
		call	fast_reference		;moving mode
        call    calc
;		call	CalcWithResult32
		bcf		dataready
		bcf		datareadyF
        bra     bpress          ;goto bpress

MainMode3 ;silent mode, only LED on/off
		btfss	dataready		;wait for data to become ready
		goto	bpress			;if data not ready goto button press detection instead
		btfss	datareadyF		;check for data to be ready
		goto	bpress			;if data not ready goto button press detection instead
                                ;else do calculations
		call	nonferro			;nonferro block, free to comment out if not needed.
		call	fast_reference		;moving mode
        call    calcLED

		bcf		dataready
		bcf		datareadyF
        bra     bpress          ;goto bpress

bpress
		banksel	PORTA
		btfsc	PORTA,push_button		;detect if button is pressed
		bra Main
        PAGESEL button
		call	button			;this button routine takes time, some mearsurements are lost while.
        bra Main

;****************************************************************
;* SUBROUTINES 			*
;****************************************************************

calc							;compare current result and reference
		movf	t_resultL,w		;copy result to temp
		movwf	tempL
		movf	t_resultH,W
		movwf	tempH

		movlw	sensitivity				;add sensitivity
		addwf	tempL,f
		clrw
		addwfc	tempH,f

		banksel	ReferenceL				;compare
		movf	tempL,w
		subwf	ReferenceL,w
		movf	tempH,w
		subwfb	ReferenceH,w
;		bcf		soundflag			;sound off or
		btfsc	STATUS,C
		bsf		soundflag			;sound on

		return
;----------------------------------------------------------------------------------------
calc32      ;gradual beeping
						;compare current result and referenc
        btfsc   soundflag   
        return              ;if soundflag already set do nothing

		banksel	ReferenceL				;Reference-result=>temp
		movf	t_resultL,w
		subwf	ReferenceL,w
        movwf   tempL
		movf	t_resultH,W
		subwfb	ReferenceH,w
        movwf   tempH
        btfss   STATUS,C
        return                  ;if Reference<result do nothing

        lslf	tempL,f         ;amplify
		rlf		tempH,f

        banksel koguja
        movfw   tempL
        addwf   koguja,f
        movfw   tempH
        addwfc  koguja+1,f
        btfss   STATUS,C
        return
        bsf     soundflag
        clrf    koguja
        comf    koguja,f
        movlw    b'11110111'
        movwf    koguja+1
        return
;----------------------------------------------------------------------------------------
CalcWithResult32							;compare current result and reference
        banksel tresultfiltered32
		movf	tresultfiltered32,w		;copy result to temp
		movwf	tempL
		movf	tresultfiltered32+1,W
		movwf	tempH

		movlw	sensitivity				;add sensitivity
		addwf	tempL,f
		clrw
		addwfc	tempH,f

		banksel	ReferenceL				;compare
		movf	tempL,w
		subwf	ReferenceL,w
		movf	tempH,w
		subwfb	ReferenceH,w
;		bcf		soundflag			;sound off or
		btfsc	STATUS,C
		bsf		soundflag			;sound on

		return
calcLED							;compare current result and reference
		movf	t_resultL,w		;copy result to temp
		movwf	tempL
		movf	t_resultH,W
		movwf	tempH

		movlw	sensitivity				;add sensitivity
		addwf	tempL,f
		clrw
		addwfc	tempH,f

		banksel	ReferenceL				;compare
		movf	tempL,w
		subwf	ReferenceL,w
		movf	tempH,w
		subwfb	ReferenceH,w
        banksel LATA
		bcf		LATA,LED1			;LED off or
		btfsc	STATUS,C
		bsf		LATA,LED1			;LED on

		return



	;------------------------------------------------------------------------------------
	;	ver3	Fast adaptive reference block for Moving Mode.
	;	Updates reference after every 64 pulses(128 msec) with maximum element of last 32 averaged sample blocks(total 4sek)(each block is average of 64samples)
	;	uses:CircularBuffer(mem block);Circularbuffer2(mem block);f_result;t_result
	;------------------------------------------------------------------------------------	
fast_reference
		;----------------------- sum = sum + new result -------------------------------
		banksel sum
		movf	t_resultL,w	
		addwf	sum,f
		movf	t_resultH,w
		addwfc	sum+1,f
		clrw
		addwfc	sum+2,f
		clrw
		addwfc	sum+3,f         ;MSB
        ;----------------------- same for  f_result (f_result2=f_result2 + new result)
        banksel f_result2
		movf	f_resultL,w
		addwf	f_result2,f
		movf	f_resultH,w
		addwfc	f_result2+1,f
		clrw
		addwfc	f_result2+2,f
		clrw
		addwfc	f_result2+3,f      ;MSB
		;------------------------------------------------------------------------------
		banksel	Counter1
		incf	Counter1,f
		btfss	Counter1,6		;bit 6 sets when counting reaches 64,decide update buffer or
		return					;not yet to update buffer
		clrf	Counter1
		;------------------------divide by 64, shift right 6 times----------------------
		banksel sum
		lsrf	sum+3,f
		rrf		sum+2,f
		rrf		sum+1,f
		rrf		sum,f

		lsrf	sum+3,f
		rrf		sum+2,f
		rrf		sum+1,f
		rrf		sum,f

		lsrf	sum+3,f
		rrf		sum+2,f
		rrf		sum+1,f
		rrf		sum,f

		lsrf	sum+3,f
		rrf		sum+2,f
		rrf		sum+1,f
		rrf		sum,f

		lsrf	sum+3,f
		rrf		sum+2,f
		rrf		sum+1,f
		rrf		sum,f

		lsrf	sum+3,f
		rrf		sum+2,f
		rrf		sum+1,f
		rrf		sum,f

        ;------------------------same for f_result------------------------------
        banksel f_result2
		lsrf	f_result2+3,f
		rrf		f_result2+2,f
		rrf		f_result2+1,f
		rrf		f_result2,f
		lsrf	f_result2+3,f
		rrf		f_result2+2,f
		rrf		f_result2+1,f
		rrf		f_result2,f
        lsrf	f_result2+3,f
		rrf		f_result2+2,f
		rrf		f_result2+1,f
		rrf		f_result2,f
        lsrf	f_result2+3,f
		rrf		f_result2+2,f
		rrf		f_result2+1,f
		rrf		f_result2,f
        lsrf	f_result2+3,f
		rrf		f_result2+2,f
		rrf		f_result2+1,f
		rrf		f_result2,f
        lsrf	f_result2+3,f
		rrf		f_result2+2,f
		rrf		f_result2+1,f
		rrf		f_result2,f
		;-----------------------update ring buffer-------------------------------------	
		banksel CircularBuffer
		movlw	high CircularBuffer
		movwf	FSR0H
		movlw	low CircularBuffer	;
		addwf	IndexW,W		;
		movwf	FSR0L			;prepare address for writing
		movf	sum,w			;
		movwf	INDF0			;write LSB value to buffer (little endian)

		incf	IndexW,f		;next address in buffer

		movf	sum+1,w			;
		movwi	++FSR0			;write MSB value to buffer and pre inc FSR0L
		
		incf	IndexW,f		;next address in buffer

		btfsc	IndexW,6		;if bit 6 is set, it has counted 64 times(32 element buffer)
		clrf	IndexW			;then reset index
                ;--------------make filtered value available as tresultfiltered32------
        banksel sum
		movf	sum,w
        banksel tresultfiltered32
        movwf   tresultfiltered32
        banksel sum+1
		movf	sum+1,w
        banksel tresultfiltered32
        movwf   tresultfiltered32+1  ;MSB
                ;---------------
        banksel sum
		clrf	sum		;do not need those any more
		clrf	sum+1			;
		clrf	sum+2			;
		clrf	sum+3
		;-----------------------update ring buffer for f_result----------------
		banksel CircularBuffer2
		movlw	high CircularBuffer2
		movwf	FSR1H
		movlw	low CircularBuffer2	;
		addwf	IndexW2,W		;
		movwf	FSR1L			;prepare address for writing
		movf	f_result2,w			;
		movwf	INDF1			;write LSB value to buffer (little endian)

		incf	IndexW2,f		;next address in buffer

		movf	f_result2+1,w			;
		movwi	++FSR1			;write MSB value to buffer and pre inc FSR0L

		incf	IndexW2,f		;next address in buffer

		btfsc	IndexW2,6		;if bit 6 is set, it has counted 64 times(32 element buffer)
		clrf	IndexW2			;then reset index
                ;--------------make filtered value available as fresultfiltered32------
        banksel f_result2
		movf	f_result2,w
        banksel fresultfiltered32
        movwf   fresultfiltered32
        banksel f_result2
		movf	f_result2+1,w
        banksel fresultfiltered32
        movwf   fresultfiltered32+1
                ;------------------------
        banksel f_result2
 		clrf	f_result2		;do not need those any more
		clrf	f_result2+1			;
		clrf	f_result2+2			;       
		;--------------ring buffer reading loop, detect maximum t_result value in buffer
		banksel CircularBuffer
		movlw	high CircularBuffer
		movwf	FSR0H
		clrf	IndexR
		movlw	low CircularBuffer	;
		movwf	FSR0L			;prepare address for reading
rloop2
		moviw	FSR0++			;read LSB value from buffer
		movwf	sum+2			;

		incf	IndexR,f		;next address

		moviw	FSR0++			;read MSB value from buffer
		movwf	sum+3			;add to sum

		incf	IndexR,f		;next address
						;--------detect biggest elemant--------------
						;--------compare old value "sum:sum+1" new value "sum+2:sum+3"------
		movf	sum+2,w
		subwf	sum,w		;sub "sum:sum+1"-"sum+2:sum+3"
		movf	sum+3,w			;compare old value to new
		subwfb	sum+1,w
		btfsc	STATUS,C		;if new value>=current maximum then copy new value
		bra     skipcopy		;else skip copy
		movf	sum+2,w
		movwf	sum
		movf	sum+3,w
		movwf	sum+1
                    ;--------also copy f_result from the same moment-----------
		movlw	high CircularBuffer2
		movwf	FSR1H
        movlw   low CircularBuffer2
        movwf	FSR1L
        decf    IndexR,w        ;-1 and to W
        addwf   FSR1L,f           ;add
        clrw
        addwfc  FSR1H,f           ;prepare address for reading
		moviw	++FSR1			;read LSB value from buffer
        movwf   fReference
        moviw   ++FSR1          
        movwf   fReference+1    ;MSB value
;        banksel CircularBuffer  ;back to this buffer
skipcopy		
		btfss	IndexR,6		;if bit 6 is set, it has counted 64 times(32 element buffer) then exit loop
		goto	rloop2			;
		clrf	IndexR
				;						new reference now in sum+1:sum
		;----------------------------------------------------------
								;update reference		
		movf	sum,w
		movwf	ReferenceL
		movf	sum+1,w
		movwf	ReferenceH

		clrf	sum		;do not need those any more
		clrf	sum+1			;
		clrf	sum+2			;
		clrf	sum+3
        ;-----------------calculate sound tone----------------------
calctone
        movf    fReference,w
        movwf   tempL
        movf    fReference+1,w
        movwf   tempH
        movlw   .5          ;ADD something
        addwf   tempL,f
        clrw
        addwfc  tempH,f
        banksel fresultfiltered32
		movf	fresultfiltered32,w       ;subtract fReference-f_result=> AARG <<8
		subwf	tempL,f             ;AARG LSB, damn divide routine is big indian
        banksel fresultfiltered32
		movf	fresultfiltered32+1,w
        subwfb  tempH,f            ;MSB
        btfss   STATUS,C    ;test if negative of zero
        bra     soundhigh
        lslf    tempL,f              ;amplify
        rlf     tempH,f
        movlw   .100          ;ADD something
        addwf   tempL,f
        clrw
        addwfc  tempH,f

        movf    tempH,w
        btfss   STATUS,Z    ;test higher byte
        bra     soundlow
        movf    tempL,w
        banksel Sound
        movwf   Sound
        return

soundhigh       ;if hi sound limit reached
        movlw   .50
        banksel Sound
        movwf   Sound
        return
soundlow       ;if lo sound limit reached
        movlw   .255
        banksel Sound
        movwf   Sound
        return

;----------------------------------------------------------------------------------------

;----------------------------------------------------------------------------------------
	;------------------------------------------------------------------------------------
	;		Moving average block 3. Uses circular buffer to calculate average of last 16 samples
	;	Input:f_resultL,f_resultH
	;	Output:f_resultL,f_resultH(updated)
	;	uses:moving_average_3(mem block)
	;------------------------------------------------------------------------------------
moving_average_3		;real time moving average of 16 measurements, simple non recursive algorithm
		banksel CircularBuffer2_3
		;---------------writing section------------------
		movlw	high	CircularBuffer2_3
		movwf	FSR0H
		movlw	low		CircularBuffer2_3	;
		addwf	IndexW2_3,W		;
		movwf	FSR0L			;prepare address for writing
		movf	f_resultL,w			;
		movwf	INDF0			;write LSB value to buffer (little endian)

		incf	IndexW2_3,f		;next address in buffer

		movf	f_resultH,w			;
		movwi	++FSR0			;write MSB value to buffer

		incf	IndexW2_3,f		;next address in buffer

		btfsc	IndexW2_3,5		;if bit 5 is set, it has counted 32
		clrf	IndexW2_3			;then reset index

		;--------------- reading section-------------------
		clrf	f_resultL
		clrf	f_resultH
		clrf	Addtmp_3
		clrf	IndexR2_3
read_avg_loop_3
		movlw	CircularBuffer2_3	;
		addwf	IndexR2_3,W		;
		movwf	FSR0L			;prepare address for reading
		movf	INDF0,W			;read LSB value from buffer
		addwf	f_resultL,f		;add to output

		incf	IndexR2_3,f		;next address

;		addfsr	FSR0,1			;prepare address for reading, add 1 to file select register address
		moviw	++FSR0			;read MSB value from buffer
		addwfc	f_resultH,f		;add to output
		clrw
		addwfc	Addtmp_3,f

		incf	IndexR2_3,f		;next address

		btfss	IndexR2_3,5		;if bit 5 is set, it has counted 32 then exit loop
		goto	read_avg_loop_3	;

		lsrf	Addtmp_3,f			;divide by 16
		rrf		f_resultH,f
		rrf		f_resultL,f

		lsrf	Addtmp_3,f
		rrf		f_resultH,f
		rrf		f_resultL,f

		lsrf	Addtmp_3,f
		rrf		f_resultH,f
		rrf		f_resultL,f

		lsrf	Addtmp_3,f
		rrf		f_resultH,f
		rrf		f_resultL,f		;ready values in t_resultL and H

	return
	;-----------------------------------------------------------------
	; Subroutine to make less sensitive to ferrite.
	;	add 0x9C00 (39936) to f_result
	;	complement f_result,
	;	add with t_result, and store in t_result
	;	RAM usage
	;Input: f_resultL, f_resultH, t_resultL, t_resultH
	;output: t_resultL, t_resultH
	;Uses: none
	;-----------------------------------------------------------------
nonferro
		movlw	0x9C
		addwf	f_resultH,f

		comf	f_resultL,w
		addwf	t_resultL,f
		comf	f_resultH,w
		addwfc	t_resultH,f
		return
;---------------------------------------------------------------------------------------
;		button press detection, when holding button down for 0,75 turns power off(Sleep)
;--------------------------------------------------------------------------------------
button					;if button is pressed for long goto sleep, if double push next mode
		;-----------------------------push type detection start----------------------------------
wait_release	;determine is it long press or short
		movlw	shortpush_max		;load wait time
		movwf	temp1
loop1
		banksel	PORTA
		btfsc	PORTA,push_button		;if button released go to double push detection
		goto	double_p
		call	_20ms
		decfsz	temp1,f
		goto	loop1
		goto 	long_push


double_p								;detect if this is double push
		call	_20ms					;debounce button release
		movlw	double_push_max-.1			;minus debounce already waited
        banksel double_p_temp
		movwf	double_p_temp
loop2
		call	_20ms
		banksel	PORTA
		btfss	PORTA,push_button
		goto	double_push
        banksel double_p_temp
		decfsz	double_p_temp,f
		goto	loop2
		goto	short_push			;if second push did not come, stick with short push
		;-----------------------------push type detection end----------------------------------

long_push
		banksel LATA
		bcf		LATA,LED1
		banksel	PORTA
		btfss	PORTA,push_button		;wait for button to be released
		goto	$-.1

sleepy  		;----------------------prepare for going to sleep-----------
		bcf		INTCON,GIE		;disable all interrupts, device will still wake from sleep
		bcf		INTCON,PEIE		;disable periferial interrupts
		bcf		INTCON,TMR0IE	;disable Timer0 interrupts

		banksel LATA
		bcf		LATA,MOSFET		;turn off mosfet before going to sleep
		bcf		LATA,Piezo1
		bcf		LATA,Piezo2

		call	_250mS			;debounce button release
		banksel	CM1CON0
		bcf		CM1CON0,C1ON	;disable comparator to save power
		banksel	DACCON0
		bcf		DACCON0,DACEN	;disable DAC voltage ref to save power
		;---------------------interrupt on change config for wake up from sleep----------------------
		banksel	IOCAN
		bsf		IOCAN,IOCAN3	;enable negative going edge interrupt on RA3
;		MOVLW	0xff			;required sequence to clear interrupt on change flags
;		XORWF	IOCAF, W		;
		movlw	0x00
		ANDWF	IOCAF, F		;clear interrupt on change flags
		bsf		INTCON,IOCIE	;enable interrupts on change		
		SLEEP
		;------------------------------SLEEP------------------------------------------------------
		nop
		bcf		INTCON,IOCIE	;disable interrupts on change
		banksel	IOCAF
		movlw	0x00
		ANDWF	IOCAF, F		;clear interrupt on change flags
        reset
		banksel	CM1CON0
		bsf		CM1CON0,C1ON	;enable comparator
		banksel	DACCON0
		bsf		DACCON0,DACEN	;enable DAC voltage ref
		call	init_buffer		;clear buffer
		call	_2ms		
		bsf		INTCON,PEIE		;enable periferial interrupts
		bsf		INTCON,TMR0IE	;enable Timer0 interrupts
		bsf		INTCON,GIE		;general interrupt enable
		call	_2ms
		call	_2ms
		banksel LATA
		bsf		LATA,LED1
		return
double_push		;-------------------double push------------------------
        banksel LATA
        bsf     LATA,LED1       ;led on
        bsf     soundflag
        incf    MainState1,f
        movlw   Maxmodes
        xorwf   MainState1,w
        btfsc   STATUS,Z    ;test if max mode reached, then back to mode0
        clrf    MainState1
        movlw   .50
        banksel Sound
        movwf   Sound
        incf    MainState1,w    ;mainstate+1 ->W
        movwf   tempL
        bsf		soundflag		;sound on
		call	_250mS
        decfsz  tempL,f          ;beep program no.
        bra     $-3
        call	_250mS
        call    init_buffer
        call	_250mS
		return
short_push		;--------------------go back to normal mode----------------------	
		banksel	TRISA
		bcf		TRISA,Piezo1	;piezo to output
		bcf		TRISA,Piezo2
		call	confTimer2		;reconfigure timer2
		return

       
	;-----------------------------------------------------------------
	;init reference buffer block. Clear all buffer contents.
	;-----------------------------------------------------------------
init_buffer
        banksel IndexR
		clrf	IndexR
		movlw	high CircularBuffer
		movwf	FSR0H
		movlw	low CircularBuffer	;
		movwf	FSR0L			;prepare first address for reading
init_loop
		clrw
		movwi	FSR0++
		incf	IndexR,f		;next address
		movwi	FSR0++
		incf	IndexR,f		;next address
		btfss	IndexR,6		;if bit 6 is set, it has counted 64 times then exit loop
		goto	init_loop			;
		clrf	IndexR
		banksel sum
		clrf	sum
		clrf	sum+1
		clrf	sum+2
        clrf	sum+3
	return
	;--------------------------------------------------------------------
    ;--------------------config Timer2 and interrupts, for sound---------
	;-----------------------------------------------------------------
confTimer2
		;start making sound
		banksel PR2
		movlw	.100            ;timer2 period
		movwf	PR2
		banksel	T2CON
		movlw	b'00000111'		;timer2 on, postscaler 1:1, prescaler 1:64
		movwf	T2CON			;timer interval 0,8ms
        banksel TMR2
		clrf	TMR2
		banksel	PIR1
		bcf		PIR1,TMR2IF		;clear timer2 interrupt flag
		bsf		INTCON,PEIE		;peripheral interrupt enable
		banksel	PIE1
		bsf		PIE1,TMR2IE		;enable timer2 interrupts
		return
confTimer2disable
		banksel	T2CON
		clrf	T2CON			;disable timer2
		banksel	PIR1
		bcf		PIR1,TMR2IF		;clear timer2 interrupt flag
		banksel	PIE1
		bcf		PIE1,TMR2IE		;disable timer2 interrupts
		return
;****************************************************************
;* Delays 			*
;****************************************************************
rtrn						;0,5 microsecond delay
	return
rtrn2						;1 microsecond delay
		goto	$+1
		goto	$+1
	return
_1ms
		movlw	.245	;2
		movwf	beeptemp	;2
looptm1	call	rtrn	; siit maalt korratakse
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn	
		decfsz	beeptemp,f	
		goto	looptm1
		return
_20ms						;20ms delay(for 32Mhz clock) debuggeris testitud
		banksel temp3
		movlw	.40			;fine tuning
		movwf	temp3
		movlw	.21			;rough tuning
		movwf	temp2
D20		call	rtrn2	
		call	rtrn2
		call	rtrn2
		call	rtrn	
		decfsz 	temp3,f
		goto 	D20
		decfsz 	temp2,f
		goto 	D20		
	return

_2ms
		movlw	.243	;2
		movwf	beeptemp	;2
looptm	call	rtrn	; siit maalt korratakse
		call	rtrn	
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn
		call	rtrn	;callisid 60 tskli jagu
		decfsz	beeptemp,f	
		goto	looptm
		return


_250mS						;250mS second delay
		banksel temp3
		movlw	.0
		movwf	temp3
		movlw	.249
		movwf	temp2
D11		call	rtrn2	
		call	rtrn2
		call	rtrn
		goto	$+1	
		decfsz 	temp3,f
		goto 	D11
		decfsz 	temp2,f
		goto 	D11		
	return


		END